@@ -23,8 +23,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "xfce4-xkb-plugin.h"
 #include "xkb-config.h"
+#include "xkb-util.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +35,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+#include <librsvg/rsvg.h>
 
 #ifndef DEBUG
 #define G_DISABLE_ASSERT
@@ -42,14 +43,21 @@
 
 typedef struct
 {
-    XklEngine*            engine;
+    gchar      *group_name;
+    gchar      *variant;
+    gchar      *pretty_layout_name;
+    GdkPixbuf  *tooltip_pixbuf;
+} t_group_data;
 
-    gchar               **group_names;
-    gchar               **variants;
-    t_xkb_settings       *settings;
+typedef struct
+{
+    XklEngine            *engine;
+
+    t_group_data         *group_data;
+
+    t_group_policy        group_policy;
     GHashTable           *variant_index_by_group;
 
-
     GHashTable           *application_map;
     GHashTable           *window_map;
 
@@ -57,18 +65,15 @@ typedef struct
     guint                 current_application_id;
 
     gint                  group_count;
+    gint                  current_group;
 
     XkbCallback           callback;
     gpointer              callback_data;
-
-
-    XklConfigRec         *config_rec;
 } t_xkb_config;
 
 static t_xkb_config *config;
-static gboolean ignore_xkl_config_change = FALSE;
 
-static void         xkb_config_state_changed            (XklEngine *engine,
+static void         xkb_config_xkl_state_changed        (XklEngine *engine,
                                                          XklEngineStateChange change,
                                                          gint group,
                                                          gboolean restore,
@@ -76,28 +81,25 @@ static void         xkb_config_state_changed            (XklEngine *engine,
 
 static void         xkb_config_xkl_config_changed       (XklEngine *engine,
                                                          gpointer user_data);
-void                xkb_config_reset_xkl_config         (XklEngine *engine,
-                                                         gpointer   user_data);
 
-static GdkFilterReturn  handle_xevent                   (GdkXEvent * xev,
+static GdkFilterReturn
+                    handle_xevent                       (GdkXEvent * xev,
                                                          GdkEvent * event);
 
 static void         xkb_config_free                     ();
-static void         xkb_config_initialize_xkb_options   (t_xkb_settings *settings);
-static gboolean     xkb_config_activate_xkl_record      ();
+static void         xkb_config_update_from_xkl          ();
+static void         xkb_config_initialize_xkb_options   (const XklConfigRec *config_rec);
 
 /* ---------------------- implementation ------------------------- */
 
 gboolean
-xkb_config_initialize (t_xkb_settings *settings,
+xkb_config_initialize (t_group_policy group_policy,
                XkbCallback callback,
                gpointer callback_data)
 {
-    g_assert (settings != NULL);
-
     config = g_new0 (t_xkb_config, 1);
 
-    config->settings = settings;
+    config->group_policy = group_policy;
 
     config->callback = callback;
     config->callback_data = callback_data;
@@ -109,7 +111,7 @@ xkb_config_initialize (t_xkb_settings *settings,
         return FALSE;
     }
 
-    xkb_config_update_settings (settings);
+    xkb_config_update_from_xkl ();
 
     xkl_engine_set_group_per_toplevel_window (config->engine, FALSE);
 
@@ -117,34 +119,79 @@ xkb_config_initialize (t_xkb_settings *settings,
 
     g_signal_connect (config->engine,
             "X-state-changed",
-            G_CALLBACK (xkb_config_state_changed),
+            G_CALLBACK (xkb_config_xkl_state_changed),
             NULL);
     g_signal_connect (config->engine,
             "X-config-changed",
             G_CALLBACK (xkb_config_xkl_config_changed),
             NULL);
-    g_signal_connect (config->engine,
-            "X-new-device",
-            G_CALLBACK (xkb_config_reset_xkl_config),
-            NULL);
     gdk_window_add_filter (NULL, (GdkFilterFunc) handle_xevent, NULL);
 
     return TRUE;
 }
 
+static gchar *
+xkb_config_xkb_description (XklConfigItem *config_item)
+{
+  gchar *ci_description;
+  gchar *description;
+
+  ci_description = g_strstrip (config_item->description);
+
+  if (ci_description[0] == 0)
+    description = g_strdup (config_item->name);
+  else
+    description = g_locale_to_utf8 (ci_description, -1, NULL, NULL, NULL);
+
+  return description;
+}
+
+static gchar*
+xkb_config_create_pretty_layout_name (XklConfigRegistry *registry,
+                                      XklConfigItem *config_item,
+                                      gchar *layout_name,
+                                      gchar *layout_variant)
+{
+    gchar *pretty_layout_name;
+
+    g_snprintf (config_item->name, sizeof (config_item->name),
+                "%s", layout_variant);
+    if (xkl_config_registry_find_variant (registry, layout_name, config_item))
+    {
+        pretty_layout_name = xkb_config_xkb_description (config_item);
+    }
+    else
+    {
+        g_snprintf (config_item->name, sizeof (config_item->name),
+                    "%s", layout_name);
+        if (xkl_config_registry_find_layout (registry, config_item))
+        {
+            pretty_layout_name = xkb_config_xkb_description (config_item);
+        }
+        else
+        {
+            pretty_layout_name = xkb_util_get_layout_string (layout_name,
+                                                             layout_variant);
+        }
+    }
+
+    return pretty_layout_name;
+}
+
 static void
-xkb_config_initialize_xkb_options (t_xkb_settings *settings)
+xkb_config_initialize_xkb_options (const XklConfigRec *config_rec)
 {
-    XklConfigRegistry *registry;
-    XklConfigItem *config_item;
     GHashTable *index_variants;
     gchar **group;
     gint val, i;
     gpointer pval;
+    gchar *imgfilename;
+    XklConfigRegistry *registry;
+    XklConfigItem *config_item;
 
     xkb_config_free ();
 
-    group = config->config_rec->layouts;
+    group = config_rec->layouts;
     config->group_count = 0;
     while (*group)
     {
@@ -154,62 +201,63 @@ xkb_config_initialize_xkb_options (t_xkb_settings *settings)
 
     config->window_map = g_hash_table_new (g_direct_hash, NULL);
     config->application_map = g_hash_table_new (g_direct_hash, NULL);
-
-    registry = xkl_config_registry_get_instance (config->engine);
-    xkl_config_registry_load (registry, FALSE);
-    g_object_unref (registry);
-
-    config_item = xkl_config_item_new ();
-
-    config->group_names = (gchar **) g_new0 (typeof (gchar **), config->group_count);
-    config->variants = (gchar **) g_new0 (typeof (gchar **), config->group_count);
+    config->group_data = (t_group_data *) g_new0 (typeof (t_group_data),
+                                                  config->group_count);
     config->variant_index_by_group = g_hash_table_new (NULL, NULL);
     index_variants = g_hash_table_new (g_str_hash, g_str_equal);
 
+    registry = xkl_config_registry_get_instance (config->engine);
+    xkl_config_registry_load (registry, FALSE);
+    config_item = xkl_config_item_new ();
+
     for (i = 0; i < config->group_count; i++)
     {
+        t_group_data *group_data = &config->group_data[i];
+        RsvgHandle *handle;
 
-        g_stpcpy (config_item->name, config->config_rec->layouts[i]);
-        config->group_names[i] = g_strdup (config->config_rec->layouts[i]);
+        group_data->group_name = g_strdup (config_rec->layouts[i]);
 
-        if (config->config_rec->variants[i] != NULL)
-        {
-            g_stpcpy (config_item->name, config->config_rec->variants[i]);
-        }
-        config->variants[i] = (config->config_rec->variants[i] == NULL)
-            ? g_strdup ("") : g_strdup (config->config_rec->variants[i]);
+        group_data->variant = (config_rec->variants[i] == NULL)
+            ? g_strdup ("") : g_strdup (config_rec->variants[i]);
 
         pval = g_hash_table_lookup (
                 index_variants,
-                config->group_names[i]
+                group_data->group_name
         );
         val = (pval != NULL) ? GPOINTER_TO_INT (pval) : 0;
         val++;
         g_hash_table_insert (
                 config->variant_index_by_group,
-                config->group_names[i],
+                GINT_TO_POINTER (i),
                 GINT_TO_POINTER (val)
         );
         g_hash_table_insert (
                 index_variants,
-                config->group_names[i],
+                group_data->group_name,
                 GINT_TO_POINTER (val)
         );
+
+        imgfilename = xkb_util_get_flag_filename (group_data->group_name);
+        handle = rsvg_handle_new_from_file (imgfilename, NULL);
+        if (handle)
+        {
+            GdkPixbuf *tmp = rsvg_handle_get_pixbuf (handle);
+            group_data->tooltip_pixbuf =
+                gdk_pixbuf_scale_simple (tmp, 30, 22, GDK_INTERP_BILINEAR);
+            g_object_unref (tmp);
+            rsvg_handle_close (handle, NULL);
+            g_object_unref (handle);
+        }
+        g_free (imgfilename);
+
+        group_data->pretty_layout_name =
+            xkb_config_create_pretty_layout_name (registry, config_item,
+                                                  group_data->group_name,
+                                                  group_data->variant);
     }
-    g_hash_table_destroy (index_variants);
     g_object_unref (config_item);
-}
-
-void
-kbd_config_free (t_xkb_kbd_config *kbd_config)
-{
-    g_free (kbd_config->model);
-    g_free (kbd_config->layouts);
-    g_free (kbd_config->variants);
-    g_free (kbd_config->toggle_option);
-    g_free (kbd_config->compose_key_position);
-
-    g_free (kbd_config);
+    g_object_unref (registry);
+    g_hash_table_destroy (index_variants);
 }
 
 static void
@@ -219,20 +267,6 @@ xkb_config_free (void)
 
     g_assert (config != NULL);
 
-    if (config->group_names)
-    {
-        for (i = 0; i < config->group_count; i++)
-            g_free (config->group_names[i]);
-        g_free (config->group_names);
-    }
-
-    if (config->variants)
-    {
-        for (i = 0; i < config->group_count; i++)
-            g_free (config->variants[i]);
-        g_free (config->variants);
-    }
-
     if (config->variant_index_by_group)
         g_hash_table_destroy (config->variant_index_by_group);
 
@@ -241,26 +275,40 @@ xkb_config_free (void)
 
     if (config->application_map)
         g_hash_table_destroy (config->application_map);
+
+    if (config->group_data)
+    {
+        for (i = 0; i < config->group_count; i++)
+        {
+            t_group_data *group_data = &config->group_data[i];
+            g_free (group_data->group_name);
+            g_free (group_data->variant);
+            g_free (group_data->pretty_layout_name);
+            if (group_data->tooltip_pixbuf)
+            {
+                g_object_unref (group_data->tooltip_pixbuf);
+            }
+        }
+        g_free (config->group_data);
+    }
 }
 
 void
 xkb_config_finalize (void)
 {
-    xkb_config_free ();
-
     xkl_engine_stop_listen (config->engine, XKLL_TRACK_KEYBOARD_STATE);
-    g_object_unref (config->config_rec);
+    g_object_unref (config->engine);
 
+    xkb_config_free ();
     g_free (config);
 
     gdk_window_remove_filter (NULL, (GdkFilterFunc) handle_xevent, NULL);
 }
 
-static gint
+gint
 xkb_config_get_current_group (void)
 {
-    XklState* state = xkl_engine_get_current_state (config->engine);
-    return state->group;
+    return config->current_group;
 }
 
 gboolean
@@ -274,6 +322,7 @@ xkb_config_set_group (gint group)
     }
 
     xkl_engine_lock_group (config->engine, group);
+    config->current_group = group;
 
     return TRUE;
 }
@@ -296,124 +345,26 @@ xkb_config_prev_group (void)
     return TRUE;
 }
 
-gboolean
-xkb_config_update_settings (t_xkb_settings *settings)
+void
+xkb_config_set_group_policy (t_group_policy group_policy)
 {
-    gboolean activate_settings = FALSE;
+    config->group_policy = group_policy;
+}
+
+static void
+xkb_config_update_from_xkl (void)
+{
+    XklConfigRec *config_rec;
 
-    gchar **opt;
-    gchar **prefix;
-    int i;
 
     g_assert (config != NULL);
-    g_assert (settings != NULL);
 
-    config->settings = settings;
+    config_rec = xkl_config_rec_new ();
+    xkl_config_rec_get_from_server (config_rec, config->engine);
 
-    if (config->config_rec == NULL)
-    {
-        config->config_rec = xkl_config_rec_new ();
-    }
+    xkb_config_initialize_xkb_options (config_rec);
 
-    if (settings->kbd_config == NULL || settings->never_modify_config)
-    {
-        xkl_config_rec_reset (config->config_rec);
-        if (!xkl_config_rec_get_from_server (config->config_rec, config->engine))
-        {
-            DBG ("ERROR: can't get xkl config: [%s]", xkl_get_last_error());
-        }
-
-        /* XklConfigRec uses for NULL for empty variant instead of "".
-           So if has skipped variants we can't get proper settings->kbd_config->variants.
-           xkl will also get confused if we try to activate() any config_rec
-           that has NULLs for empty variants.
-           Hence, we'll replace NULL variants with ""
-        */
-        for (i = 0; config->config_rec->layouts[i]; i++)
-        {
-            if (!config->config_rec->variants[i])
-                config->config_rec->variants[i] = g_strdup ("");
-        }
-
-        XKB_DEBUG_CONFIG_REC (config->config_rec, "retrieved");
-        if (settings->kbd_config == NULL)
-            settings->kbd_config = g_new0 (t_xkb_kbd_config, 1);
-
-        g_free (settings->kbd_config->model);
-        settings->kbd_config->model = g_strdup (config->config_rec->model);
-        g_free (settings->kbd_config->layouts);
-        settings->kbd_config->layouts = g_strjoinv (",", config->config_rec->layouts);
-        g_free (settings->kbd_config->variants);
-        settings->kbd_config->variants = g_strjoinv (",", config->config_rec->variants);
-    }
-    else
-    {
-        gchar *options;
-
-        activate_settings = TRUE;
-
-        g_free (config->config_rec->model);
-        config->config_rec->model = g_strdup (settings->kbd_config->model ? settings->kbd_config->model : "");
-
-        g_strfreev (config->config_rec->layouts);
-        config->config_rec->layouts = g_strsplit_set (settings->kbd_config->layouts, ",", 0);
-
-        g_strfreev (config->config_rec->variants);
-        config->config_rec->variants = g_strsplit_set (settings->kbd_config->variants, ",", 0);
-
-        if (settings->kbd_config->toggle_option
-                && strlen (settings->kbd_config->toggle_option) > 0)
-            options = g_strdup (settings->kbd_config->toggle_option);
-        else options = g_strdup ("");
-
-        if (settings->kbd_config->compose_key_position
-                && strlen (settings->kbd_config->compose_key_position) > 0)
-        {
-            gchar *tmp = options;
-            options = g_strconcat (tmp, ",", settings->kbd_config->compose_key_position, NULL);
-            g_free (tmp);
-        }
-
-        g_strfreev (config->config_rec->options);
-        config->config_rec->options = g_strsplit_set (options, ",", 0);
-        g_free (options);
-    }
-
-    /* select the first "grp" option and use it (should be fixed to support more options) */
-    g_free (settings->kbd_config->toggle_option);
-    settings->kbd_config->toggle_option = NULL;
-    g_free (settings->kbd_config->compose_key_position);
-    settings->kbd_config->compose_key_position = NULL;
-    opt = config->config_rec->options;
-    while (opt && *opt)
-    {
-        prefix = g_strsplit(*opt, ":", 2);
-        if (settings->kbd_config->toggle_option == NULL
-                && prefix && *prefix && strcmp(*prefix, "grp") == 0)
-        {
-            settings->kbd_config->toggle_option = g_strdup (*opt);
-        }
-        else if (prefix && *prefix && strcmp(*prefix, "compose") == 0)
-        {
-            settings->kbd_config->compose_key_position = g_strdup (*opt);
-        }
-
-        g_strfreev (prefix);
-        opt++;
-    }
-
-    if (activate_settings && !settings->never_modify_config)
-    {
-        if (!xkb_config_activate_xkl_record ())
-        {
-            // FIXME: we should probably try to reload from xkl config here
-            return FALSE;
-        }
-    }
-
-    xkb_config_initialize_xkb_options (settings);
-
-    return TRUE;
+    g_object_unref (config_rec);
 }
 
 void
@@ -429,7 +380,7 @@ xkb_config_window_changed (guint new_window_id, guint application_id)
     id = 0;
     hashtable = NULL;
 
-    switch (config->settings->group_policy)
+    switch (config->group_policy)
     {
         case GROUP_POLICY_GLOBAL:
             return;
@@ -447,18 +398,18 @@ xkb_config_window_changed (guint new_window_id, guint application_id)
             break;
     }
 
-    group = config->settings->default_group;
+    group = 0;
 
     if (g_hash_table_lookup_extended (hashtable, GINT_TO_POINTER (id), &key, &value))
     {
         group = GPOINTER_TO_INT (value);
     }
-
-    g_hash_table_insert (
-            hashtable,
-            GINT_TO_POINTER (id),
-            GINT_TO_POINTER (group)
-    );
+    else
+    {
+        g_hash_table_insert (hashtable,
+                             GINT_TO_POINTER (id),
+                             GINT_TO_POINTER (group));
+    }
 
     xkb_config_set_group (group);
 }
@@ -468,7 +419,7 @@ xkb_config_application_closed (guint application_id)
 {
     g_assert (config != NULL);
 
-    switch (config->settings->group_policy)
+    switch (config->group_policy)
     {
         case GROUP_POLICY_GLOBAL:
         case GROUP_POLICY_PER_WINDOW:
@@ -489,7 +440,7 @@ xkb_config_window_closed (guint window_id)
 {
     g_assert (config != NULL);
 
-    switch (config->settings->group_policy)
+    switch (config->group_policy)
     {
         case GROUP_POLICY_GLOBAL:
         case GROUP_POLICY_PER_APPLICATION:
@@ -525,7 +476,7 @@ xkb_config_get_group_name (gint group)
     if (group == -1)
         group = xkb_config_get_current_group ();
 
-    return config->group_names[group];
+    return config->group_data[group].group_name;
 }
 
 const gchar*
@@ -539,19 +490,21 @@ xkb_config_get_variant (gint group)
     if (group == -1)
         group = xkb_config_get_current_group ();
 
-    return config->variants[group];
+    return config->group_data[group].variant;
 }
 
 void
-xkb_config_state_changed (XklEngine *engine,
-                          XklEngineStateChange change,
-                          gint group,
-                          gboolean restore,
-                          gpointer user_data)
+xkb_config_xkl_state_changed (XklEngine *engine,
+                              XklEngineStateChange change,
+                              gint group,
+                              gboolean restore,
+                              gpointer user_data)
 {
     if (change == GROUP_CHANGED)
     {
-        switch (config->settings->group_policy)
+        config->current_group = group;
+
+        switch (config->group_policy)
         {
             case GROUP_POLICY_GLOBAL:
                 break;
@@ -580,13 +533,7 @@ xkb_config_state_changed (XklEngine *engine,
 void
 xkb_config_xkl_config_changed (XklEngine *engine, gpointer user_data)
 {
-    TRACE ("ignore_xkl_config_change: %d ", ignore_xkl_config_change);
-
-    if (ignore_xkl_config_change) return;
-
-    kbd_config_free (config->settings->kbd_config);
-    config->settings->kbd_config = NULL;
-    xkb_config_update_settings (config->settings);
+    xkb_config_update_from_xkl ();
 
     if (config->callback != NULL)
     {
@@ -595,31 +542,19 @@ xkb_config_xkl_config_changed (XklEngine *engine, gpointer user_data)
     }
 }
 
-void
-xkb_config_reset_xkl_config (XklEngine *engine,
-                             gpointer   user_data)
-{
-    TRACE ("X-new-device: %d", config->settings->never_modify_config);
-    if (!config->settings->never_modify_config)
-        xkb_config_activate_xkl_record ();
-}
-
 gint
 xkb_config_variant_index_for_group (gint group)
 {
     gpointer presult;
     gint result;
-    gchar *key;
 
     g_return_val_if_fail (config != NULL, 0);
 
     if (group == -1) group = xkb_config_get_current_group ();
 
-    key = config->group_names[group];
-
     presult = g_hash_table_lookup (
             config->variant_index_by_group,
-            key
+            GINT_TO_POINTER (group)
     );
     if (presult == NULL) return 0;
 
@@ -651,34 +586,14 @@ xkb_config_get_xkl_registry (void)
     return registry;
 }
 
-gint
-xkb_config_get_max_layout_number (void)
+GdkPixbuf*
+xkb_config_get_tooltip_pixbuf (gint group)
 {
-    if (config == NULL) return 0;
-
-    return xkl_engine_get_max_num_groups (config->engine);
+    return config->group_data[group].tooltip_pixbuf;
 }
 
-gboolean
-xkb_config_activate_xkl_record (void)
+gchar*
+xkb_config_get_pretty_layout_name (gint group)
 {
-    gboolean rc;
-
-    if (config->config_rec->model == NULL)
-    {
-        config->config_rec->model = g_strdup ("");
-    }
-
-    XKB_DEBUG_CONFIG_REC (config->config_rec, "activate");
-
-    ignore_xkl_config_change = TRUE;
-    rc = xkl_config_rec_activate (config->config_rec, config->engine);
-    ignore_xkl_config_change = FALSE;
-
-    if (!rc)
-    {
-        DBG ("ERROR: can't activate xkl config: [%s]", xkl_get_last_error());
-    }
-
-    return rc;
+    return config->group_data[group].pretty_layout_name;
 }
