@@ -32,6 +32,7 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <gdk/gdk.h>
 #include <libxklavier/xklavier.h>
 
 typedef enum
@@ -41,33 +42,15 @@ typedef enum
     GROUP_POLICY_PER_APPLICATION    = 2
 } t_group_policy;
 
-typedef struct
-{
-    gchar*          model;
-    gchar*          layouts;
-    gchar*          variants;
-    gchar*          toggle_option;
-    gchar*          compose_key_position;
-} t_xkb_kbd_config;
-
-typedef struct
-{
-    t_group_policy      group_policy;
-    gint                default_group;
-    gboolean            never_modify_config;
-    t_xkb_kbd_config*   kbd_config;
-} t_xkb_settings;
-
 typedef void        (*XkbCallback)                  (gint current_group,
                                                      gboolean groups_changed,
                                                      gpointer user_data);
 
-gboolean          xkb_config_initialize                   (t_xkb_settings *settings,
+gboolean          xkb_config_initialize                   (t_group_policy group_policy,
                                                            XkbCallback callback,
                                                            gpointer data);
 void              xkb_config_finalize                     (void);
-void              kbd_config_free                         (t_xkb_kbd_config *kbd_config);
-gboolean          xkb_config_update_settings              (t_xkb_settings *settings);
+void              xkb_config_set_group_policy             (t_group_policy group_policy);
 gint              xkb_config_get_group_count              (void);
 const gchar*      xkb_config_get_group_name               (gint group);
 const gchar*      xkb_config_get_variant                  (gint group);
@@ -80,47 +63,43 @@ void              xkb_config_window_changed               (guint new_window_id,
                                                            guint application_id);
 void              xkb_config_application_closed           (guint application_id);
 void              xkb_config_window_closed                (guint window_id);
+GdkPixbuf*        xkb_config_get_tooltip_pixbuf           (gint group);
+gchar*            xkb_config_get_pretty_layout_name       (gint group);
+gint              xkb_config_get_current_group            (void);
 
 /* TODO: remove this function - xkl structures should not be used outside xkb-config */
 XklConfigRegistry*
                   xkb_config_get_xkl_registry             (void);
-gint              xkb_config_get_max_layout_number        (void);
 
 #ifdef DEBUG
 
 #define XKB_DEBUG(...) \
     do { g_fprintf (stderr, "[[ XFCE XKB PLUGIN ]]: "__VA_ARGS__); g_fprintf (stderr, "\n"); } while (0)
 
-#define XKB_DEBUG_KBD(kbd, msg) g_printf("DUMPING KEYBOARD SETTINGS [[[%s]]] {%p}: ", msg, kbd);\
+#define XKB_DEBUG_KBD(kbd, msg) g_printf("DUMPING KEYBOARD SETTINGS [[[%s]]] {%d}: ", msg, kbd);\
     if (kbd) { \
         g_printf ("\n\
-          model: %s [%p]\n\
-          layouts: %s [%p]\n\
-          variants: %s [%p]\n\
-          toggle: %s [%p]\n\
-          compose_key: %s [%p]\n", \
+          model: %s [%d]\n\
+          layouts: %s [%d]\n\
+          variants: %s [%d]\n\
+          options: %s [%d]\n", \
                 kbd->model, kbd->model, \
                 kbd->layouts, kbd->layouts, \
                 kbd->variants, kbd->variants, \
-                kbd->toggle_option, kbd->toggle_option,   \
-                kbd->compose_key_position, kbd->compose_key_position); \
+                kbd->options, kbd->options); \
     } else { \
         g_printf("NULL\n"); \
     }
 
-#define XKB_DEBUG_CONFIG_REC(crec, msg) g_printf("DUMPING CONFIG REC [[[%s]]] {%p}: ", msg, crec);\
+#define XKB_DEBUG_CONFIG_REC(crec, msg) g_printf("DUMPING CONFIG REC [[[%s]]] {%d}: ", msg, (int) crec);\
     if (crec) { \
-        gchar *layouts = g_strjoinv (",", crec->layouts); \
-        gchar *variants = g_strjoinv (",", crec->variants); \
         g_printf ("\n\
-            model: %s [%p]\n\
-            layouts: %s [%p]\n\
-            variants: %s [%p]\n", \
-                crec->model, crec->model, \
-                layouts, crec->layouts, \
-                variants, crec->variants); \
-        g_free (layouts); \
-        g_free (variants); \
+            model: %s [%d]\n\
+            layouts: %s [%d]\n\
+            variants: %s [%d]\n", \
+                crec->model, (int) crec->model, \
+                g_strjoinv (",", crec->layouts), (int) crec->layouts, \
+                g_strjoinv (",", crec->variants), (int) crec->variants); \
     } else { \
         g_printf ("NULL\n"); \
     }
